@@ -15,7 +15,7 @@
     (c) 2014, Uday Verma, Hobu, Inc.
 
     This is free software; you can redistribute and/or modify it under the
-    terms of the GNU Lesser General Licence as published by the Free Software
+    terms of the Apache Public License 2.0 published by the Apache Software
     Foundation. See the COPYING file for more information.
 
     This software is distributed WITHOUT ANY WARRANTY and without even the
@@ -165,6 +165,7 @@ TEST(io_tests, parses_laszip_vlr_correctly)
         EXPECT_EQ(vlr.items[0].version, 2);
     };
 
+    // Test deprecated functions.
     check(vlr);
     std::vector<char> vd = vlr.data();
     laz_vlr vlr2(vd.data());
@@ -636,6 +637,22 @@ TEST(io_tests, issue22)
     std::remove(tempfile.c_str());
 }
 
+TEST(io_tests, vlrIndex)
+{
+    reader::named_file f(testFile("point10.las.laz"));
+
+    std::vector<char> vlr = f.vlrData("LASF_Projection", 34735);
+    EXPECT_EQ(vlr.size(), 72u);
+    vlr = f.vlrData("Foo", 34735);
+    EXPECT_EQ(vlr.size(), 0u);
+    vlr = f.vlrData("LASF_Projection", 25);
+    EXPECT_EQ(vlr.size(), 0u);
+    vlr = f.vlrData("LASF_Projection", 34737);
+    EXPECT_EQ(vlr.size(), 28u);
+    vlr = f.vlrData("liblas", 2112);
+    EXPECT_EQ(vlr.size(), 640u);
+}
+
 /**
 TEST(io_tests, fast)
 {
@@ -650,4 +667,100 @@ TEST(io_tests, fast)
 }
 **/
 
+namespace
+{
+
+void pre_1_2_common_header_checks(const base_header& header)
+{
+  EXPECT_EQ(header.global_encoding, 0);
+  EXPECT_EQ(header.version.major, 1);
+  EXPECT_EQ(header.creation.day, 200);
+  EXPECT_EQ(header.creation.year, 2023);
+  EXPECT_EQ(header.header_size, 227);
+  EXPECT_EQ(header.point_format_id, 0);
+  EXPECT_EQ(header.point_record_length, 20);
+  EXPECT_DOUBLE_EQ(header.scale.x, 1.0);
+  EXPECT_DOUBLE_EQ(header.scale.y, 1.0);
+  EXPECT_DOUBLE_EQ(header.scale.z, 1.0);
+  EXPECT_DOUBLE_EQ(header.offset.x, 0.0);
+  EXPECT_DOUBLE_EQ(header.offset.y, 0.0);
+  EXPECT_DOUBLE_EQ(header.offset.z, 0.0);
+  EXPECT_DOUBLE_EQ(header.minx, 1.0);
+  EXPECT_DOUBLE_EQ(header.miny, 2.0);
+  EXPECT_DOUBLE_EQ(header.minz, 3.0);
+  EXPECT_DOUBLE_EQ(header.maxx, 1.0);
+  EXPECT_DOUBLE_EQ(header.maxy, 2.0);
+  EXPECT_DOUBLE_EQ(header.maxz, 3.0);
+  EXPECT_EQ(header.point_count, 1);
+}
+
+void pre_1_2_common_point_checks(const las::point10& p10)
+{
+  EXPECT_DOUBLE_EQ(p10.x, 1.0);
+  EXPECT_DOUBLE_EQ(p10.y, 2.0);
+  EXPECT_DOUBLE_EQ(p10.z, 3.0);
+  EXPECT_EQ(p10.intensity, 42);
+  EXPECT_EQ(p10.return_number, 1);
+  EXPECT_EQ(p10.number_of_returns_of_given_pulse, 1);
+  EXPECT_EQ(p10.classification, 5);
+  EXPECT_EQ(p10.scan_angle_rank, 7);
+}
+
+}
+
+TEST(io_tests, can_read_1_0)
+{
+  for (const std::string filename : { "point10-1.0.las", "point10-1.0.laz" })
+  {
+    reader::named_file f(testFile(filename));
+    const base_header& header = f.header();
+    pre_1_2_common_header_checks(header);
+    EXPECT_EQ(header.version.minor, 0);
+    EXPECT_EQ(header.global_encoding, 0);
+
+    las::point10 p10;
+    f.readPoint(reinterpret_cast<char*>(&p10));
+    pre_1_2_common_point_checks(p10);
+    EXPECT_EQ(p10.user_data, 0);
+  }
+}
+
+TEST(io_tests, can_read_1_1)
+{
+  for (const std::string filename : { "point10-1.1.las", "point10-1.1.laz" })
+  {
+    reader::named_file f(testFile(filename));
+    const base_header& header = f.header();
+    pre_1_2_common_header_checks(header);
+    EXPECT_EQ(header.version.minor, 1);
+    EXPECT_EQ(header.file_source_id, 1000);
+
+    las::point10 p10;
+    f.readPoint(reinterpret_cast<char*>(&p10));
+    pre_1_2_common_point_checks(p10);
+    EXPECT_EQ(p10.user_data, 13);
+  }
+}
+
+TEST(io_tests, can_open_no_points_file)
+{
+    for (const std::string filename : { "no-points-1.3.las", "no-points-1.3.laz" })
+    {
+        reader::named_file f(testFile(filename));
+        EXPECT_EQ(f.header().point_count, 0);
+    }
+}
+
+TEST(io_tests, write_laz_with_no_points_and_read_back)
+{
+    const auto file_name = makeTempFileName();
+
+    const writer::named_file::config c({ 0.01, 0.01, 0.01 }, { 0.0, 0.0, 0.0 });
+    writer::named_file out(file_name, c);
+    out.close();
+
+    reader::named_file f(file_name);
+    EXPECT_EQ(f.header().point_count, 0);
+}
+
 } // namespace lazperf
